/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_VIDEO_IOCTL_H__
#define __CX88_VIDEO_IOCTL_H__

#include "cx88_common_ioctl.h"

#define CX88_VIDEO_CHANNEL_Y   0
#define CX88_VIDEO_CHANNEL_U   1
#define CX88_VIDEO_CHANNEL_V   2
#define CX88_VIDEO_CHANNEL_VBI 3
#define CX88_VIDEO_CHANNELS    4

#define CX88_VIDEO_FIELD_EVEN(channel) (1 << (2 * channel))
#define CX88_VIDEO_FIELD_ODD(channel)  (1 << ((2 * channel) + 1))

typedef struct
{  
   /* Number of frames to allocate */
   #define CX88_VIDEO_MAX_FRAMES 32
   uint8_t in_num_frames;
   
   uint8_t in_one_shot;   
      
   /* Buffer configuration */
   struct
   {
      union
      {
         struct
         {
            /* mmap() works at page granularity, so to acommodate page-rounding, 
            * applications will need to use out_mmap_size to determine the total number
            * of bytes to mmap(), and out_buffer_size to compute buffer offsets within
            * the mmap'ed region. */
            uint32_t out_buffer_size; 
            uint32_t out_mmap_size;
         } out_kernelbuf; /* Kernel-mode buffer allocation */
         struct
         {
            void *buffers[CX88_VIDEO_MAX_FRAMES]; /* User-mode buffer allocation */
         } in_userbuf; 
      } u;
     
      uint16_t in_x_header;
      uint16_t in_x_trailer;
      uint16_t in_y_header;
      uint16_t in_y_trailer;
      
      #define CX88_VIDEO_BUF_KERNEL 0
      #define CX88_VIDEO_BUF_USER 1
      uint8_t in_buftype; /* Select user- or kernel-mode buffers */

   } bufconfig[CX88_VIDEO_CHANNELS];
} cx88_video_config;

typedef struct
{
   uint8_t enable;
   
   #define CX88_VIDEO_SWAP_WORDS (1 << 0)
   #define CX88_VIDEO_SWAP_BYTES (1 << 1)
   uint8_t swap;

} cx88_video_field;

typedef struct
{
   #define CX88_VIDEO_FMT_NTSCM        1
   #define CX88_VIDEO_FMT_NTSCJ        2
   #define CX88_VIDEO_FMT_NTSC443      3
   #define CX88_VIDEO_FMT_PALBDGHI     4
   #define CX88_VIDEO_FMT_PALM         5
   #define CX88_VIDEO_FMT_PALN         6
   #define CX88_VIDEO_FMT_PALNC        7
   #define CX88_VIDEO_FMT_PAL60        8
   #define CX88_VIDEO_FMT_SECAM        9
   uint8_t in_video_format;
   
   #define CX88_VIDEO_RGB32      0
   #define CX88_VIDEO_BGR24      1
   #define CX88_VIDEO_RGB16      2
   #define CX88_VIDEO_RGB15      3
   #define CX88_VIDEO_YUV422     4
   #define CX88_VIDEO_BTYUV411   5
   #define CX88_VIDEO_Y8         6
   #define CX88_VIDEO_TGB8       7
   #define CX88_VIDEO_YCC422     8
   #define CX88_VIDEO_YCC411     9
   uint8_t in_pixel_format;
   
   uint16_t in_rows;
   uint16_t in_cols;
   
   uint16_t in_x_offset;
   uint16_t in_x_size;
   
   uint16_t in_y_offset;
   uint16_t in_y_size;
   
   uint8_t in_enable_vbi;
   
   cx88_video_field in_even_field;
   cx88_video_field in_odd_field;
   
   uint8_t in_interlace;
   
   struct
   {
      uint16_t line_size;
      uint16_t num_lines;
   } out_geoms[CX88_VIDEO_CHANNELS];
   
} cx88_video_format;

typedef struct
{   
   #define CX88_VIDEO_INPUT_MUX1             0
   #define CX88_VIDEO_INPUT_MUX2             1
   #define CX88_VIDEO_INPUT_MUX3             2
   #define CX88_VIDEO_INPUT_MUX4             3
   uint8_t in_input_source;
   
   #define CX88_VIDEO_INPUT_COMPOSITE        0
   #define CX88_VIDEO_INPUT_SVIDEO           1
   #define CX88_VIDEO_INPUT_VIP              2
   uint8_t in_input_type;
   
   #define CX88_VIDEO_INPUT_AGC_ENABLE       (1 << 0)
   #define CX88_VIDEO_INPUT_AGC_COARSE       (1 << 1) 
   #define CX88_VIDEO_INPUT_AGC_ADAPTIVE     (1 << 2)
   #define CX88_VIDEO_INPUT_COLORKILLER      (1 << 3)
   uint8_t in_input_options;
   
   uint16_t in_lock_timeout_ms;
   uint16_t in_lock_delay_ms;
   
   uint8_t in_no_frame_intrs;
} cx88_video_capture;

typedef struct
{
   uint32_t in_timeout_ms;  /* Maximum time to wait (0 -> no timeout) */
   
   uint8_t in_channel_mask;
   
   /* Capture status */
   #define CX88_VIDEO_CAP_NOERR     0
   #define CX88_VIDEO_CAP_FATAL     (1 << 0)
   #define CX88_VIDEO_CAP_TIMEOUT   (1 << 1)
   #define CX88_VIDEO_SYNC_ERROR    ((1 << 2) | CX88_VIDEO_CAP_FATAL)
   #define CX88_VIDEO_BUS_ERROR     ((1 << 3) | CX88_VIDEO_CAP_FATAL)
   #define CX88_VIDEO_DMA_PROGRAM   ((1 << 4) | CX88_VIDEO_CAP_FATAL)
   #define CX88_VIDEO_WAIT_ABORTED  ((1 << 5) | CX88_VIDEO_CAP_FATAL)
   uint8_t out_status;
      
   struct
   {
      uint8_t out_even_frame_available;
      uint8_t out_odd_frame_available;
      uint8_t overflow;
   } channels[CX88_VIDEO_CHANNELS];
} cx88_video_wait;

typedef struct
{
   uint8_t out_video_present;
   
   #define CX88_VIDEO_MODE_SP       0
   #define CX88_VIDEO_MODE_NORMAL   1
   uint8_t out_play_mode;
   
   uint8_t out_horizontal_lock;
   
   #define CX88_VIDEO_SYSTEM_525    0
   #define CX88_VIDEO_SYSTEM_625    1
   uint8_t out_system_type;
   
} cx88_video_status;

typedef struct
{
   uint16_t in_start_line;
   uint16_t in_end_line;
   
   #define CX88_CLIP_INCLUDE 0
   #define CX88_CLIP_EXCLUDE 1
   uint8_t in_clip_type;
   
   uint8_t in_num_points;
   uint16_t in_points[];
} cx88_video_clip_pattern;

typedef struct
{
   struct
   {
      void *in_patterns;
      uint32_t in_pattern_size;
   } channels[CX88_VIDEO_CHANNELS];
} cx88_video_clip;

#define CX88_VIDEO_FORMAT      _IOWR('V', 0, cx88_video_format)
#define CX88_VIDEO_CONFIG      _IOWR('V', 1, cx88_video_config)
#define CX88_VIDEO_CAPTURE     _IOW('V', 2, cx88_video_capture)
#define CX88_VIDEO_STOP        _IO('V', 3)
#define CX88_VIDEO_WAIT        _IOWR('V', 4, cx88_video_wait)
#define CX88_VIDEO_HUE         _IOW('V', 5, uint8_t)
#define CX88_VIDEO_BRIGHTNESS  _IOW('V', 6, uint8_t)
#define CX88_VIDEO_CONTRAST    _IOW('V', 7, uint8_t)
#define CX88_VIDEO_SAT_U       _IOW('V', 8, uint8_t)
#define CX88_VIDEO_SAT_V       _IOW('V', 9, uint8_t)
#define CX88_VIDEO_DROP_FRAMES _IOW('V', 10, uint8_t)
#define CX88_VIDEO_GET_STATUS  _IOR('V', 11, cx88_video_status)
#define CX88_VIDEO_CLIP        _IOW('V', 12, cx88_video_clip)

#endif
